module.exports = PanoramaModule = {
    Panorama: {
        // Global variables to store configuration
        coordinates: null,
        iFrameRequestUrl: null,
        panoramaContainer: null,
        iFrameElement: null,
        panoramaClickCheckBox: null,
        closeButton: null,
        keyboardMovementEnabled: null,
        mapClickEnabled: null,
        markerImage: null,
        mapLibreMap: null,
        maplibreMarker: null,
        // Variable to store whether the panorama is loaded or not
        panoramaLoaded: false,
        orientation: null,
        ttRight: null,
        ttBottom: null,
        /**
         * @param {number} x The x coordinate, given in epsg:4326
         * @param {number} y The y coordinate, given in epsg:4326
         */
        // Opens the panorama at the location x,y
        OpenPanoramaOnLocation: function (x, y, orientation) {
            // If panorama is closed, open it
            if (PanoramaModule.Panorama.panoramaContainer.style.display == 'none') {
                // Open panorama
                PanoramaModule.Panorama.OpenPanoramaFrame([x, y], orientation);
                // If panorama is not loaded yet
                if (PanoramaModule.Panorama.panoramaLoaded === false) {
                    // Repeat this function until panorama is loaded
                    timer = setTimeout(function () {
                        PanoramaModule.Panorama.OpenPanoramaOnLocation(x, y, orientation);
                    }, 50);
                }
            }
            // If panorama is not loaded yet
            else if (PanoramaModule.Panorama.panoramaLoaded === false) {
                // Repeat this function until panorama is loaded
                timer = setTimeout(function () {
                    PanoramaModule.Panorama.OpenPanoramaOnLocation(x, y, orientation);
                }, 50);
            }
            // If panorama is loaded
            else {
                // If keyboard movement is enabled
                if (PanoramaModule.Panorama.keyboardMovementEnabled == true) {
                    // Send this message once
                    PanoramaModule.Panorama.keyboardMovementEnabled = false;
                    PanoramaModule.Panorama.iFrameElement.contentWindow.postMessage({ keyboardEnabled: true }, '*');
                }
                if (PanoramaModule.Panorama.ttRight) {
                    PanoramaModule.Panorama.iFrameElement.contentWindow.postMessage({ ttRight: true }, '*');
                }
                if (PanoramaModule.Panorama.ttBottom) {
                    PanoramaModule.Panorama.iFrameElement.contentWindow.postMessage({ ttBottom: true }, '*');
                }
                // When the frame is loaded post message to open new coordinates
                PanoramaModule.Panorama.iFrameElement.contentWindow.postMessage({ coordinates: [x, y], orientation: orientation }, '*');
            }
        },
        // Opens the panorama iframe
        OpenPanoramaFrame: function (coordinates, orientation) {
            // If source empty fill url
            if (PanoramaModule.Panorama.iFrameElement.src == window.location.href || PanoramaModule.Panorama.iFrameElement.src == undefined) {
                PanoramaModule.Panorama.iFrameElement.src = `${PanoramaModule.Panorama.iFrameRequestUrl}&coordinates=${coordinates[0]},${coordinates[1]}`;
                if (orientation) {
                    if (orientation.yaw) {
                        PanoramaModule.Panorama.iFrameElement.src += `&yaw=${orientation.yaw}`;
                    }
                    if (orientation.pitch) {
                        PanoramaModule.Panorama.iFrameElement.src += `&pitch=${orientation.pitch}`;
                    }
                    if (orientation.fov) {
                        PanoramaModule.Panorama.iFrameElement.src += `&fov=${orientation.fov}`;
                    }
                }
            }
            // If src full send message
            else {
                // When the frame is loaded post message to open new coordinates
                PanoramaModule.Panorama.iFrameElement.contentWindow.postMessage({ coordinates: coordinates, orientation: orientation }, '*');
            }
            // Unhide it
            PanoramaModule.Panorama.panoramaContainer.style.display = '';
            // Add event listener to close button
            PanoramaModule.Panorama.closeButton.addEventListener('click', PanoramaModule.Panorama.HidePanoramaFrame);
            // Get the marker
            const marker = document.getElementById('panorama-view-cone');
            // If there is a marker
            if (marker != null) {
                // Enable marker
                marker.style.display = '';
            }
        },
        // Hides the panorama frame
        HidePanoramaFrame: function (event) {
            // Hide container
            PanoramaModule.Panorama.panoramaContainer.style.display = 'none';
            // Get the marker
            const marker = document.getElementById('panorama-view-cone');
            // If there is a marker
            if (marker != null) {
                // Remove marker
                marker.style.display = 'none';
            }
        },
        /**
         * @param {Object} map The map object, after initialized using: new maplibregl.Map
         * @param {Object} maplibregl The maplibregl object that is imported
         */
        // Configure MapLibre map for panorama usage, call this method in map.onload
        ConfigureMapLibreMap: function (map, maplibregl) {
            PanoramaModule.Panorama.mapLibreMap = map;
            // If keyboard movement is on
            if (PanoramaModule.Panorama.keyboardMovementEnabled == true) {
                // Disable map keyboard controls
                map.keyboard.disable();
            }
            // If map is clickable
            if (PanoramaModule.Panorama.mapClickEnabled != false) {
                // Open panorama on click location
                map.on('click', PanoramaModule.Panorama.MapLibreClickEvent);
            }
            // If marker is enabled (exists)
            if (PanoramaModule.Panorama.markerImage != null) {
                // Create new marker
                const newMarker = document.createElement('div');
                newMarker.id = 'panorama-view-cone';
                const img = new Image();
                img.addEventListener("load", function () {
                    newMarker.style.width = img.naturalWidth + 'px';
                    newMarker.style.height = img.naturalHeight + 'px';
                    newMarker.style.backgroundImage = 'url(' + img.src + ')';
                });
                img.src = PanoramaModule.Panorama.markerImage;
                PanoramaModule.Panorama.maplibreMarker = new maplibregl.Marker(newMarker);
                // When new location or angle comes as message
                window.addEventListener('message', PanoramaModule.Panorama.MapLibreMessageEvent)
            }
        },
        // Event for maplibre map click
        MapLibreClickEvent: function (e) {
            // If checkbox is checked
            if (PanoramaModule.Panorama.panoramaClickCheckBox.checked == true) {
                // Open panorama on location
                PanoramaModule.Panorama.OpenPanoramaOnLocation(e.lngLat.lng, e.lngLat.lat);
            }
        },
        // Event for maplibre event-marker
        MapLibreMessageEvent: function (e) {
            if (e.data.angle != undefined && e.data.coordinates != undefined) {
                // If panorama is open
                if (PanoramaModule.Panorama.panoramaContainer.style.display != 'none') {
                    // Get the marker
                    const marker = document.getElementById('panorama-view-cone');
                    // If there is a marker
                    if (marker != null) {
                        // Get its coordinates
                        const markerCoordinates = PanoramaModule.Panorama.maplibreMarker.getLngLat();
                        // If the new coordinates are different than the markers coordinates
                        if (markerCoordinates && markerCoordinates.lng != parseFloat(e.data.coordinates[0]) && markerCoordinates.lat != parseFloat(e.data.coordinates[1])) {
                            // Center map
                            PanoramaModule.Panorama.coordinates = e.data.coordinates;
                            PanoramaModule.Panorama.mapLibreMap.flyTo({ center: e.data.coordinates });
                        }
                    }
                    else {
                        PanoramaModule.Panorama.maplibreMarker.setLngLat([29, 41]).addTo(PanoramaModule.Panorama.mapLibreMap);
                    }
                    PanoramaModule.Panorama.maplibreMarker.setLngLat(e.data.coordinates).setRotation(e.data.angle);
                }
            }
        },
        /**
         * @param {String} properties.token The request token
         * @param {String} properties.environment The mode that the panorama will run on, default is deploy, test and development also available
         * @param {String} properties.panoramaType Default: 2013-2018 - The type of panorama, empty string for 2013-2018, 2018 for 2018 and 2013 for 2013
         * @param {String} properties.panoramaContainerId Default: panorama-container - The container that includes the panorama and its related controls
         * @param {String} properties.panoramaIframeId Default: panorama-iframe - Id of the iframe element
         * @param {String} properties.panoramaCloseButtonId Default: panorama-close-button - Id of the button that closes the panorama container
         * @param {Boolean} properties.mapClickEnabled Default: false - The parameter that decides whether a click on the map will open the panorama, loads default checkbox with id: "panorama-checkbox"
         * @param {String} properties.panoramaCheckBoxElementId The checkbox that enables "click map open panorama".
         * @param {String} properties.markerImage Default: https://cbspanorama.ibb.gov.tr/images/panoiconBlue.png - The image for the view cone
         * @param {Boolean} properties.keyboardControlsEnabled Default: true - Enables or disables keyboard movement inside the panorama.
         * @param {object} properties.orientation {yaw: number, pitch: number, fov: number}
         * @param {double[]} properties.coordinates {array with [x,y]}
         * @param {Boolean} properties.timeTravelRight {If time travel is to the right}
         * @param {Boolean} properties.timeTravelBottom {If time travel is to the bottom}
        */
        // Configure panorama
        ConfigurePanorama: function (properties) {
            // Set panoramaType to empty string if null
            if (properties.panoramaType === undefined) {
                properties.panoramaType = '';
            }
            //#region environment
            // Set request url according to env
            switch (properties.environment) {
                case "development":
                    PanoramaModule.Panorama.iFrameRequestUrl = 'https://localhost:44314/panorama/panorama' + properties.panoramaType + '/?token=' + properties.token;
                    break;
                case "test":
                    PanoramaModule.Panorama.iFrameRequestUrl = 'https://cbspanoramatest.ibb.gov.tr/panorama/panorama' + properties.panoramaType + '/?token=' + properties.token;
                    break;
                default:
                    PanoramaModule.Panorama.iFrameRequestUrl = 'https://cbspanorama.ibb.gov.tr/panorama/panorama' + properties.panoramaType + '/?token=' + properties.token;
            }
            //#endregion
            //#region  panoraa container
            // Get panorama container from document
            switch (properties.panoramaContainerId) {
                case undefined:
                    PanoramaModule.Panorama.panoramaContainer = document.getElementById('panorama-container');
                    break;
                default:
                    PanoramaModule.Panorama.panoramaContainer = document.getElementById(properties.panoramaContainerId);
            }
            // Hide container
            PanoramaModule.Panorama.panoramaContainer.style.display = 'none';
            //#endregion
            //#region panorama iframe
            // Get panorama iframe
            switch (properties.panoramaIframeId) {
                case undefined:
                    PanoramaModule.Panorama.iFrameElement = document.getElementById('panorama-iframe');
                    break;
                default:
                    PanoramaModule.Panorama.iFrameElement = document.getElementById(properties.panoramaIframeId);
            }
            //#endregion
            //#region close button
            // Get close button from document
            switch (properties.panoramaCloseButtonId) {
                case undefined:
                    PanoramaModule.Panorama.closeButton = document.getElementById('panorama-close-button');
                    break;
                default:
                    PanoramaModule.Panorama.closeButton = document.getElementById(properties.panoramaCloseButtonId);
            }
            //#endregion
            //#region map click 
            switch (properties.mapClickEnabled) {
                case true:
                    PanoramaModule.Panorama.mapClickEnabled = properties.mapClickEnabled;
                    switch (properties.panoramaClickCheckBox) {
                        case undefined:
                            PanoramaModule.Panorama.panoramaClickCheckBox = document.getElementById('panorama-checkbox');
                            break;
                        default:
                            PanoramaModule.Panorama.panoramaClickCheckBox = document.getElementById(properties.panoramaCheckBoxElementId);
                    }
                    switch (properties.mode) {
                        case "development":
                            PanoramaModule.Panorama.markerImage = 'https://localhost:44314/images/panoiconBlue.png';
                            break;
                        case "test":
                            PanoramaModule.Panorama.markerImage = 'https://cbspanoramatest.ibb.gov.tr/images/panoiconBlue.png';
                            break;
                        default:
                            PanoramaModule.Panorama.markerImage = 'https://cbspanorama.ibb.gov.tr/images/panoiconBlue.png';
                    }
                    break;
                default:
                    PanoramaModule.Panorama.mapClickEnabled = false;
            }
            //#endregion
            //#region marker image
            // If marker image is specified
            switch (properties.markerImage) {
                case undefined:
                    switch (properties.environment) {
                        case "development":
                            PanoramaModule.Panorama.markerImage = 'https://localhost:44314/images/panoiconBlue.png';
                            break;
                        case "test":
                            PanoramaModule.Panorama.markerImage = 'https://cbspanoramatest.ibb.gov.tr/images/panoiconBlue.png';
                            break;
                        default:
                            PanoramaModule.Panorama.markerImage = 'https://cbspanorama.ibb.gov.tr/images/panoiconBlue.png';
                    }
                    break;
                default:
                    PanoramaModule.Panorama.markerImage = properties.markerImage;
            }
            switch (properties.keyboardMovementEnabled) {
                case undefined:
                    PanoramaModule.Panorama.keyboardMovementEnabled = true;
                    break;
                default:
                    PanoramaModule.Panorama.keyboardMovementEnabled = properties.keyboardControlsEnabled;
            }
            if (properties.timeTravelRight) {
                PanoramaModule.Panorama.ttRight = properties.timeTravelRight;
            }
            if (properties.timeTravelBottom) {
                PanoramaModule.Panorama.ttBottom = properties.timeTravelBottom;
            }
            //#endregion
            // Catch the panorama loaded message
            window.addEventListener('message', PanoramaModule.Panorama.MessageEvent);
        },
        // Event to catch load
        MessageEvent: function (e) {
            if (e.data.load === true) {
                PanoramaModule.Panorama.panoramaLoaded = true;
            }
            if (e.data.angle && e.data.pitch && e.data.fov) {
                PanoramaModule.Panorama.orientation = { yaw: e.data.angle, pitch: e.data.pitch, fov: e.data.fov }
            }
        },
        GetOrientation() {
            return PanoramaModule.Panorama.orientation;
        },
        GetCoordinates() {
            return PanoramaModule.Panorama.coordinates;
        },
        // Resets this class
        Reset: function () {
            // Remove event listener
            window.removeEventListener('message', PanoramaModule.Panorama.MessageEvent);
            // If maplibre
            if (PanoramaModule.Panorama.mapLibreMap !== null) {
                window.removeEventListener('message', PanoramaModule.Panorama.MapLibreMessageEvent);
                const marker = document.getElementById('panorama-view-cone');
                if (marker != null) {
                    marker.parentElement.removeChild(marker);
                }
            }
            PanoramaModule.Panorama.iFrameElement.src = "";
            PanoramaModule.Panorama.iFrameRequestUrl = null;
            PanoramaModule.Panorama.panoramaContainer = null;
            PanoramaModule.Panorama.iFrameElement = null;
            PanoramaModule.Panorama.panoramaClickCheckBox = null;
            PanoramaModule.Panorama.closeButton = null;
            PanoramaModule.Panorama.keyboardMovementEnabled = null;
            PanoramaModule.Panorama.mapClickEnabled = null;
            PanoramaModule.Panorama.markerImage = null;
            PanoramaModule.Panorama.mapLibreMap = null;
            PanoramaModule.Panorama.maplibreMarker = null;
            PanoramaModule.Panorama.orientation = null;
        }
    }
}